@@ -18,7 +18,6 @@
  */
 
 #include "php_swoole_cxx.h"
-#include "php_swoole_thread.h"
 #include "php_swoole_coroutine_system.h"
 
 #include "swoole_server.h"
@@ -45,6 +44,8 @@ END_EXTERN_C()
  */
 #define SW_RECOVER_CANCELED_EXCEPTION 0
 
+#define INVALID_PTR -1
+
 using std::unordered_map;
 using swoole::Coroutine;
 using swoole::PHPContext;
@@ -414,12 +415,13 @@ void PHPCoroutine::activate() {
         interrupt_thread_start();
     }
 
-    if (config.hook_flags) {
-        enable_hook(config.hook_flags);
+    if (sw_is_main_thread()) {
+        if (config.hook_flags) {
+            enable_hook(config.hook_flags);
+        }
+        disable_unsafe_function();
     }
 
-    disable_unsafe_function();
-
     /* deactivate when reactor free */
     sw_reactor()->add_destroy_callback(deactivate, nullptr);
     Coroutine::activate();
@@ -437,10 +439,11 @@ void PHPCoroutine::deactivate(void *ptr) {
     }
     activated = false;
     interrupt_thread_stop();
-    /**
-     * reset runtime hook
-     */
-    disable_hook();
+
+    if (sw_is_main_thread()) {
+        disable_hook();
+        enable_unsafe_function();
+    }
 
     Coroutine::set_on_yield(nullptr);
     Coroutine::set_on_resume(nullptr);
@@ -452,7 +455,6 @@ void PHPCoroutine::deactivate(void *ptr) {
         deadlock_check();
     }
 
-    enable_unsafe_function();
     Coroutine::deactivate();
 }
 
@@ -467,6 +469,9 @@ void PHPCoroutine::shutdown() {
     free_main_context();
 }
 
+// !!! [Danger Warning]
+// The deadlock_check function can only print information and cannot execute any async functions
+// This function can only execute internal code and cannot provide any callbacks, allowing the execution of user code
 void PHPCoroutine::deadlock_check() {
     if (Coroutine::count() == 0) {
         return;
@@ -707,7 +712,7 @@ void PHPCoroutine::destroy_context(PHPContext *ctx) {
     // Release resources
     if (ctx->context) {
         zend_object *context = ctx->context;
-        ctx->context = (zend_object *) ~0;
+        ctx->context = reinterpret_cast<zend_object *>(INVALID_PTR);
         OBJ_RELEASE(context);
     }
 
@@ -734,7 +739,7 @@ void PHPCoroutine::destroy_context(PHPContext *ctx) {
 
 void PHPCoroutine::main_func(void *_args) {
     bool exception_caught = false;
-    Args *args = (Args *) _args;
+    const auto args = static_cast<Args *>(_args);
     PHPContext *ctx = create_context(args);
 
     zend_first_try {
@@ -838,10 +843,10 @@ void PHPCoroutine::defer(zend::Function *fci) {
 
 void PHPCoroutine::fiber_context_init(PHPContext *ctx) {
     auto *fiber_context = static_cast<zend_fiber_context *>(emalloc(sizeof(zend_fiber_context)));
-    fiber_context->handle = (void *) -1;
-    fiber_context->kind = (void *) -1;
-    fiber_context->function = (zend_fiber_coroutine) -1;
-    fiber_context->stack = (zend_fiber_stack *) -1;
+    fiber_context->handle = reinterpret_cast<void *>(INVALID_PTR);
+    fiber_context->kind = reinterpret_cast<void *>(INVALID_PTR);
+    fiber_context->function = reinterpret_cast<zend_fiber_coroutine>(INVALID_PTR);
+    fiber_context->stack = reinterpret_cast<zend_fiber_stack *>(INVALID_PTR);
     ctx->fiber_context = fiber_context;
 
     zend_observer_fiber_init_notify(fiber_context);
@@ -1043,13 +1048,6 @@ void php_swoole_coroutine_minit(int module_number) {
     SW_REGISTER_LONG_CONSTANT("SWOOLE_EXIT_IN_COROUTINE", SW_EXIT_IN_COROUTINE);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_EXIT_IN_SERVER", SW_EXIT_IN_SERVER);
 
-    /* hook autoload */
-    original_zend_autoload = zend_autoload;
-    zend_autoload = swoole_coroutine_autoload;
-    SWOOLE_G(in_autoload) = nullptr;
-}
-
-void php_swoole_coroutine_rinit() {
     if (SWOOLE_G(cli)) {
 #if PHP_VERSION_ID < 80400
         ori_exit_handler = zend_get_user_opcode_handler(ZEND_EXIT);
@@ -1063,6 +1061,13 @@ void php_swoole_coroutine_rinit() {
         zend_set_user_opcode_handler(ZEND_END_SILENCE, coro_end_silence_handler);
     }
 
+    /* hook autoload */
+    original_zend_autoload = zend_autoload;
+    zend_autoload = swoole_coroutine_autoload;
+    SWOOLE_G(in_autoload) = nullptr;
+}
+
+void php_swoole_coroutine_rinit() {
     PHPCoroutine::init_main_context();
 }
 
@@ -1181,9 +1186,9 @@ static PHP_METHOD(swoole_coroutine, getContext) {
         swoole_set_last_error(SW_ERROR_CO_NOT_EXISTS);
         RETURN_NULL();
     }
-    if (UNEXPECTED(ctx->context == reinterpret_cast<zend_object *>(~0))) {
+    if (UNEXPECTED(ctx->context == reinterpret_cast<zend_object *>(INVALID_PTR))) {
         /* bad context (has been destroyed), see: https://github.com/swoole/swoole-src/issues/2991 */
-        php_swoole_fatal_error(E_WARNING, "Context of this coroutine has been destroyed");
+        php_swoole_fatal_error(E_WARNING, "The context of coroutine has been destroyed");
         RETURN_NULL();
     }
     if (UNEXPECTED(!ctx->context)) {
