@@ -172,6 +172,7 @@ struct ServerPortGS {
 };
 
 struct ListenPort {
+    uint16_t object_id;
     /**
      * tcp socket listen backlog
      */
@@ -967,6 +968,10 @@ class Server {
         return get_port_by_fd(conn->fd);
     }
 
+    uint32_t get_package_max_length(Connection *conn) {
+        return get_port_by_fd(conn->fd)->protocol.package_max_length;
+    }
+
     network::Socket *get_server_socket(int fd) const {
         return connection_list[fd].socket;
     }
